@@ -184,7 +184,7 @@ panel.parallel <-
 parallel <- function(x, data, ...) UseMethod("parallel")
 
 ## {
-##     ocall <- match.call()
+##     ocall <- sys.call(sys.parent())
 ##     formula <- ocall$formula
 ##     if (!is.null(formula))
 ##     {
@@ -200,7 +200,8 @@ parallel.matrix <-
 parallel.data.frame <-
     function(x, data = NULL, ...)
 {
-    ocall <- ccall <- match.call()
+    ocall <- sys.call(sys.parent())
+    ccall <- match.call()
     if (!is.null(ccall$data)) 
         warning("explicit 'data' specification ignored")
     ccall$data <- list(x = x)
@@ -309,7 +310,7 @@ parallel.formula <-
 
     dots <- foo$dots # arguments not processed by trellis.skeleton
     foo <- foo$foo
-    foo$call <- match.call()
+    foo$call <- sys.call(sys.parent())
 
     ## Step 2: Compute scales.common (leaving out limits for now)
 
